package es.uca.xmleye.xpath;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;

/**
 * Implementación del comportamiento mínimo establecido por
 * java.xml.namespace.NamespaceContext.
 * 
 * Para ser fácilmente reutilizable, incorpora tablas hash en ambas direcciones
 * para hacer la correspondencia. No permite redefinir los prefijos xml ni
 * xmlns, ni sus URI.
 * 
 * Suponemos que un URI sólo puede tener un prefijo (getPrefix y getPrefixes son
 * iguales).
 * 
 * No se admite el prefijo con la cadena vacía (prefijo por defecto), ya que
 * realmente NO se aplica al usar XPath (un elemento sin prefijo es un elemento
 * sin namespace).
 * 
 * @author Antonio García Domínguez
 * @version 1.0
 */
public class HashNamespaceContext implements NamespaceContext {
	private Map<String, String> prefijoURI = new HashMap<String, String>();
	private Map<String, String> uriPrefijo = new HashMap<String, String>();

	public String getNamespaceURI(String prefix) {
		if (prefix.equals(XMLConstants.XML_NS_PREFIX))
			return XMLConstants.XML_NS_URI;
		else if (prefix.equals(XMLConstants.XMLNS_ATTRIBUTE))
			return XMLConstants.XMLNS_ATTRIBUTE_NS_URI;
		else if (prefix == null)
			throw new IllegalArgumentException();
		else if (prefijoURI.containsKey(prefix))
			return prefijoURI.get(prefix);
		else
			return XMLConstants.NULL_NS_URI;
	}

	public String getPrefix(String uri) {
		if (uri.equals(XMLConstants.XML_NS_URI))
			return XMLConstants.XML_NS_PREFIX;
		else if (uri.equals(XMLConstants.XMLNS_ATTRIBUTE_NS_URI))
			return XMLConstants.XMLNS_ATTRIBUTE;
		else if (uri == null)
			throw new IllegalArgumentException();
		else if (uriPrefijo.containsKey(uri))
			return uriPrefijo.get(uri);
		else
			return XMLConstants.DEFAULT_NS_PREFIX;
	}

	@SuppressWarnings("unchecked")
	public Iterator getPrefixes(String uri) {
		String[] elems = new String[] { getPrefix(uri) };
		return Arrays.asList(elems).iterator();
	}

	/**
	 * Establece la URI para un prefijo.
	 * 
	 * @param prefix
	 *            Prefijo a usar (puede ser la cadena vacía)
	 * @param uri
	 *            URI a usar (puede ser la cadena vacía)
	 * @exception IllegalArgumentException
	 *                Se ha pasado un prefijo nulo, vacío, o reservado (xml o
	 *                xmlns), o una uri nula.
	 */
	public void set(String prefix, String uri) {
		if (prefix == null || prefix.length() == 0 || uri == null
				|| prefix.equals(XMLConstants.XML_NS_PREFIX)
				|| prefix.equals(XMLConstants.XMLNS_ATTRIBUTE)) {
			throw new IllegalArgumentException();
		}

		uriPrefijo.put(uri, prefix);
		prefijoURI.put(prefix, uri);
	}
}